"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getRequestValidatorPluginDirective = exports.generateOperationPlugins = exports.generateGlobalPlugins = exports.generateRequestValidatorPlugin = exports.generateBodyOptions = exports.ALLOW_ALL_SCHEMA = exports.generatePlugins = exports.isRequestValidatorPluginKey = void 0;
const swagger_parser_1 = __importDefault(require("@apidevtools/swagger-parser"));
const common_1 = require("../common");
const kong_1 = require("../types/kong");
const isRequestValidatorPluginKey = (property) => (property.match(/-request-validator$/) != null);
exports.isRequestValidatorPluginKey = isRequestValidatorPluginKey;
function generatePlugins(item, tags) {
    // When generating plugins, ignore the request validator plugin because it is handled at the operation level
    const pluginFilter = ([key]) => (0, common_1.isPluginKey)(key) && !(0, exports.isRequestValidatorPluginKey)(key);
    // Server plugins should load from the api spec root and from the server
    return Object.entries(item)
        .filter(pluginFilter)
        .map(generatePlugin(tags));
}
exports.generatePlugins = generatePlugins;
const generatePlugin = (tags) => ([key, value]) => {
    var _a;
    return ({
        ...(value !== null && value !== void 0 ? value : {}),
        name: value.name || (0, common_1.getPluginNameFromKey)(key),
        tags: [
            // Add tags to plugins while appending defaults tags
            ...tags,
            ...((_a = value.tags) !== null && _a !== void 0 ? _a : []),
        ],
    });
};
/**
 * This is valid config to allow all content to pass
 * See: https://github.com/Kong/kong-plugin-enterprise-request-validator/pull/34/files#diff-1a1d2d5ce801cc1cfb2aa91ae15686d81ef900af1dbef00f004677bc727bfd3cR284
 */
exports.ALLOW_ALL_SCHEMA = '{}';
const $schema = 'http://json-schema.org/schema#';
const DEFAULT_PARAM_STYLE = {
    header: 'simple',
    cookie: 'form',
    query: 'form',
    path: 'simple',
};
const resolveParameter = ($refs, parameter) => {
    if ('$ref' in parameter) {
        const dereferenced = getOperationRef($refs, parameter.$ref);
        const { $ref, ...param } = parameter;
        let schema = dereferenced === null || dereferenced === void 0 ? void 0 : dereferenced.schema;
        if (schema && '$ref' in schema) {
            schema = getOperationRef($refs, schema.$ref);
        }
        const resolvedParam = {
            ...param,
            ...dereferenced,
            name: (dereferenced === null || dereferenced === void 0 ? void 0 : dereferenced.name) || '',
            in: (dereferenced === null || dereferenced === void 0 ? void 0 : dereferenced.in) || '',
            schema,
        };
        const components = resolveComponents($refs, resolvedParam);
        return {
            resolvedParam,
            components,
        };
    }
    if (parameter.schema && '$ref' in parameter.schema) {
        const schema = getOperationRef($refs, parameter.schema.$ref);
        const resolvedParam = { ...parameter, schema };
        const components = resolveComponents($refs, resolvedParam);
        return {
            resolvedParam,
            components,
        };
    }
    return { resolvedParam: parameter, components: undefined };
};
const generateParameterSchema = async (api, operation) => {
    var _a, _b;
    if (!((_a = operation === null || operation === void 0 ? void 0 : operation.parameters) === null || _a === void 0 ? void 0 : _a.length)) {
        return;
    }
    const refs = await swagger_parser_1.default.resolve(api);
    const parameterSchemas = [];
    for (const parameter of operation.parameters) {
        // The following is valid config to allow all content to pass, in the case where schema is not defined
        let schema = '';
        const { resolvedParam, components } = resolveParameter(refs, parameter);
        if (resolvedParam.schema) {
            const kongSchema = { ...resolvedParam.schema };
            // The $schema property should only exist if components exist with a $ref path
            if (components) {
                kongSchema.components = components;
                kongSchema.$schema = $schema;
            }
            schema = JSON.stringify(kongSchema);
        }
        else if ('content' in parameter) {
            // only parameters defined with a schema (not content) are supported
            schema = exports.ALLOW_ALL_SCHEMA;
        }
        else {
            // no schema or content property on a parameter is in violation with the OpenAPI spec
            schema = exports.ALLOW_ALL_SCHEMA;
        }
        // @ts-expect-error fix this
        const paramStyle = (_b = parameter.style) !== null && _b !== void 0 ? _b : DEFAULT_PARAM_STYLE[resolvedParam.in];
        if (typeof paramStyle === 'undefined') {
            const name = resolvedParam.name;
            throw new Error(`invalid 'in' property (parameter '${name}')`);
        }
        const parameterSchema = {
            in: resolvedParam.in,
            explode: !!resolvedParam.explode,
            required: !!resolvedParam.required,
            name: resolvedParam.name,
            schema,
            style: paramStyle,
        };
        parameterSchemas.push(parameterSchema);
    }
    return parameterSchemas;
};
function resolveRequestBodyContent($refs, operation) {
    if (!operation || !(operation === null || operation === void 0 ? void 0 : operation.requestBody)) {
        return;
    }
    if ('$ref' in operation.requestBody) {
        return getOperationRef($refs, operation.requestBody.$ref);
    }
    return operation.requestBody;
}
function getOperationRef($refs, refPath) {
    if ($refs.exists(refPath)) {
        return $refs.get(refPath);
    }
    return;
}
/**
 * Resolves a ref for the given schema recursively with unknown types.
 * @param $refs SwaggerParser.$Ref object to get free methods
 * @param source the source object to be parsed by resolving all the references
 * @param components Map object to map each component path key and value recursively
 * @returns a New Map object that capture all resolved refs of path keys and values upto the Nth iteration.
 */
function resolveRefSchemaRecursively($refs, source, components = new Map()) {
    const componentsRefMap = new Map([...components]);
    if (typeof source !== 'object' || source === null) {
        // if the source is any non-object literal (string, boolean, number, undefined) we want to bail because you cannot iterate these values. (Since `typeof null` equals `'object'` in JavaScript, we also want to catch null)
        return componentsRefMap;
    }
    if (Array.isArray(source)) {
        // if the source is an array literal, we want to just return it literally (i.e. skip) because all we're looking for is the `$ref` value (which is always an object in the OpenAPI spec). For the purposes of this function, we don't care about arrays because it means the value isn't relevant to what this function tries to do (which is, resolving `$ref`s).
        return componentsRefMap;
    }
    return Object.entries(source)
        .reduce((acc, [key, value]) => {
        if (typeof value === 'string') {
            // the responsiblity of this function is only really concerning `$ref`s, so if the end value is not a `$ref` we can skip it
            if (key !== '$ref') {
                return acc;
            }
            // the accumulator alreadly having this `$ref` value indicates that we have already visited this ref, and as such we should exit early to prevent an infinite loop since circular `$ref`s are valid in OpenAPI.
            if (acc.has(value)) {
                return acc;
            }
            const pathResolved = getOperationRef($refs, value);
            if (pathResolved) {
                acc.set(value, pathResolved);
                const resolved = resolveRefSchemaRecursively($refs, pathResolved, acc);
                return new Map([...acc, ...resolved]);
            }
            return acc;
        }
        // if the value is not iterable (i.e. array or object) we want to bail
        if (typeof value !== 'object') {
            return acc;
        }
        // since `typeof null` equals `object` in JavaScript, we catch this case separately
        if (value === null) {
            return acc;
        }
        if (Array.isArray(value)) {
            // if the array is empty, we're done
            if (value.length === 0) {
                return acc;
            }
            // resolve the refs (recursively) looking for `$ref`s
            return value.reduce((newAcc, item) => (resolveRefSchemaRecursively($refs, item, newAcc)), new Map([...acc]));
        }
        // iterate into the next level in the recursion with `value` as the source (value is now narrowed to an Object)
        return new Map([...acc, ...resolveRefSchemaRecursively($refs, value, acc)]);
    }, componentsRefMap);
}
/**
 * Build components reference object with nested paths
 * @param mapObject Map object that keeps all the ref keys and valuese to be transformed into the final object product with nested paths
 * @returns final object product with nested paths
 */
function buildComponentsObjectFromMap(mapObject) {
    return Array
        .from(mapObject.entries())
        .reduce((acc, [pathKey, pathValue]) => {
        const paths = pathKey.replace('#/components/', '').split('/');
        const lastPath = paths.pop();
        if (lastPath) {
            paths.reduce((r, a) => r[a] = r[a] || {}, acc)[lastPath] = pathValue;
        }
        return acc;
    }, {});
}
/**
 * Resolves a set of components used in the given schema recursively
 * @param $refs SwaggerParser.$Ref object to get free methods
 * @param schema schema object to be recursively resolved
 * @returns OpenAPIV3 component objects completely dereferenced for all paths mentioned in the given schema
 */
function resolveComponents($refs, schema) {
    const componentsMap = resolveRefSchemaRecursively($refs, schema);
    if (!componentsMap.size) {
        return;
    }
    const components = buildComponentsObjectFromMap(componentsMap);
    return components;
}
/**
 * Serializes schema used in the Kong configuration parameter schema or body schema with optional JSON properties for reference resolving.
 * @param schema parsed Object to be passed to the Kong configuration either in parameter schema or body schema
 * @param components component Object to be referred during Kong configuration parsing (outside of Insomnia/Inso context)
 * @returns Object with the schema and JSON schema properties
 */
function serializeSchemaForKong(schema, components) {
    const kongSchema = { ...schema };
    // we probably want to include 'components' and '$schema' only if 'components' exists
    if (components) {
        kongSchema.components = components;
        kongSchema.$schema = $schema;
    }
    return JSON.stringify(kongSchema);
}
function resolveItemSchema($refs, item) {
    var _a;
    if (item.schema && '$ref' in item.schema) {
        const schema = getOperationRef($refs, item.schema.$ref);
        if (schema) {
            const components = resolveComponents($refs, schema);
            return { schema, components };
        }
    }
    const hasNoRef = { schema: (_a = item.schema) !== null && _a !== void 0 ? _a : {}, components: undefined };
    return hasNoRef;
}
async function generateBodyOptions(api, operation) {
    const $refs = await swagger_parser_1.default.resolve(api);
    let bodySchema;
    let allowedContentTypes;
    const requestBody = resolveRequestBodyContent($refs, operation);
    const bodyContent = requestBody === null || requestBody === void 0 ? void 0 : requestBody.content;
    if (bodyContent) {
        const jsonContentType = 'application/json';
        allowedContentTypes = Object.keys(bodyContent);
        if (allowedContentTypes.includes(jsonContentType)) {
            const item = bodyContent[jsonContentType];
            const { schema, components } = resolveItemSchema($refs, item);
            for (const key in schema.properties) {
                // Append 'null' to property type if nullable true, seeccccc
                if (schema.properties[key].nullable === true) {
                    // @ts-expect-error this needs some further investigation. 'type' is merely an string literal union, not an array (i.e. tuple) according to the OpenAPI 3 typings for `SchemaObject.type`.
                    schema.properties[key].type = [schema.properties[key].type, 'null'];
                }
            }
            bodySchema = serializeSchemaForKong(schema, components);
        }
    }
    return {
        bodySchema,
        allowedContentTypes,
    };
}
exports.generateBodyOptions = generateBodyOptions;
async function generateRequestValidatorPlugin({ tags, api, plugin = { name: 'request-validator' }, operation, }) {
    var _a, _b, _c, _d, _e;
    const config = {
        version: 'draft4',
    };
    // // Use original or generated parameter_schema
    const parameterSchema = (0, kong_1.isParameterSchema)(plugin.config) ? plugin.config.parameter_schema : await generateParameterSchema(api, operation);
    const generated = await generateBodyOptions(api, operation);
    // Use original or generated body_schema
    let bodySchema = (0, kong_1.isBodySchema)(plugin.config) ? plugin.config.body_schema : generated.bodySchema;
    // If no parameter_schema or body_schema is defined or generated, allow all content to pass
    if (parameterSchema === undefined && bodySchema === undefined) {
        bodySchema = exports.ALLOW_ALL_SCHEMA;
    }
    // Apply parameter_schema and body_schema to the config object
    if (parameterSchema !== undefined) {
        config.parameter_schema = parameterSchema;
    }
    if (bodySchema !== undefined) {
        config.body_schema = bodySchema;
    }
    // Use original or generated allowed_content_types
    const allowedContentTypes = (_b = (_a = plugin.config) === null || _a === void 0 ? void 0 : _a.allowed_content_types) !== null && _b !== void 0 ? _b : generated.allowedContentTypes;
    if (allowedContentTypes !== undefined) {
        config.allowed_content_types = allowedContentTypes;
    }
    // Use original verbose_response if defined
    if (((_c = plugin.config) === null || _c === void 0 ? void 0 : _c.verbose_response) !== undefined) {
        config.verbose_response = Boolean(plugin.config.verbose_response);
    }
    const isEnabledSpecified = Object.prototype.hasOwnProperty.call(plugin, 'enabled');
    const enabled = isEnabledSpecified ? { enabled: Boolean((_d = plugin.enabled) !== null && _d !== void 0 ? _d : true) } : {};
    const requestValidatorPlugin = {
        name: 'request-validator',
        config: config,
        tags: [
            ...(tags !== null && tags !== void 0 ? tags : []),
            ...((_e = plugin.tags) !== null && _e !== void 0 ? _e : []),
        ],
        ...enabled,
    };
    return requestValidatorPlugin;
}
exports.generateRequestValidatorPlugin = generateRequestValidatorPlugin;
async function generateGlobalPlugins(api, tags) {
    const globalPlugins = generatePlugins(api, tags);
    const plugin = (0, exports.getRequestValidatorPluginDirective)(api);
    if (plugin) {
        globalPlugins.push(await generateRequestValidatorPlugin({ plugin, tags, api }));
    }
    return {
        // Server plugins take precedence over global plugins
        plugins: (0, common_1.distinctByProperty)(globalPlugins, plugin => plugin.name),
        requestValidatorPlugin: plugin,
    };
}
exports.generateGlobalPlugins = generateGlobalPlugins;
const generateOperationPlugins = async ({ operation, pathPlugins, parentValidatorPlugin, tags, api }) => {
    const operationPlugins = generatePlugins(operation, tags);
    // Check if validator plugin exists on the operation, even if the value of the plugin is undefined
    const operationValidatorPlugin = (0, exports.getRequestValidatorPluginDirective)(operation);
    // Use the operation or parent validator plugin, or skip if neither exist
    const plugin = operationValidatorPlugin || parentValidatorPlugin;
    if (plugin) {
        operationPlugins.push(await generateRequestValidatorPlugin({ plugin, tags, operation, api }));
    }
    // Operation plugins take precedence over path plugins
    return (0, common_1.distinctByProperty)([...operationPlugins, ...pathPlugins], plugin => plugin.name);
};
exports.generateOperationPlugins = generateOperationPlugins;
/** This function accepts any OpenAPI3 document segment that can have the request validator plugin and returns it if found */
const getRequestValidatorPluginDirective = (segment) => {
    const key = Object.keys(segment).filter(common_1.isPluginKey).find(exports.isRequestValidatorPluginKey);
    // If the key is defined but is blank (therefore should be fully generated) then default to {}, which is truthy.
    return key ? (segment[key] || {}) : undefined;
};
exports.getRequestValidatorPluginDirective = getRequestValidatorPluginDirective;
//# sourceMappingURL=plugins.js.map